package java.util.jar;

import java.io.IOException;
import java.io.BufferedOutputStream;
import java.util.zip.ZipEntry;
import java.io.OutputStream;
import java.util.zip.ZipOutputStream;

public class JarOutputStream extends ZipOutputStream
{
    private static final int JAR_MAGIC = 51966;
    private boolean firstEntry;
    
    public JarOutputStream(OutputStream out, Manifest manifest) throws IOException {
        super(out);
        this.firstEntry = true;
        if (manifest == null) {
            throw new NullPointerException("man");
        }
        this.putNextEntry(new ZipEntry("META-INF/MANIFEST.MF"));
        manifest.write(new BufferedOutputStream(this));
        this.closeEntry();
        return;
    }
    
    public JarOutputStream(OutputStream out) throws IOException {
        super(out);
        this.firstEntry = true;
    }
    
    @Override
    public void putNextEntry(ZipEntry e) throws IOException {
        if (this.firstEntry) {
            byte[] extra = e.getExtra();
            if (extra == null || !hasMagic(extra)) {
                byte[] extra2;
                if (extra == null) {
                    extra2 = new byte[4];
                }
                else {
                    byte[] array = new byte[extra.length + 4];
                    System.arraycopy(extra, 0, array, 4, extra.length);
                    extra2 = array;
                }
                set16(extra2, 0, 51966);
                set16(extra2, 2, 0);
                e.setExtra(extra2);
            }
            this.firstEntry = false;
        }
        super.putNextEntry(e);
    }
    
    private static boolean hasMagic(byte[] array) {
        try {
            int i = 0;
            while (i < array.length) {
                if (get16(array, i) == 51966) {
                    return true;
                }
                i += get16(array, i + 2) + 4;
            }
        }
        catch (ArrayIndexOutOfBoundsException ex) {}
        return false;
    }
    
    private static int get16(byte[] array, int n) {
        return Byte.toUnsignedInt(array[n]) | Byte.toUnsignedInt(array[n + 1]) << 8;
    }
    
    private static void set16(byte[] array, int n, int n2) {
        array[n + 0] = (byte)n2;
        array[n + 1] = (byte)(n2 >> 8);
    }
}
